@@ -131,13 +131,14 @@ SEXP R_PDGETRI(SEXP A, SEXP CLDIM, SEXP DESCA, SEXP N)
 
 
 // -------------------------------------------------------- 
-// Auxillary 
+// Factorizations 
 // -------------------------------------------------------- 
 
 
 /* SVD */
 SEXP R_PDGESVD(SEXP M, SEXP N, SEXP ASIZE, SEXP A, SEXP DESCA, SEXP ALDIM, 
-  SEXP ULDIM, SEXP DESCU, SEXP VTLDIM, SEXP DESCVT, SEXP JOBU, SEXP JOBVT)
+  SEXP ULDIM, SEXP DESCU, SEXP VTLDIM, SEXP DESCVT, SEXP JOBU, SEXP JOBVT, 
+  SEXP INPLACE)
 {
   int i, *pt_ALDIM = INTEGER(ALDIM);
   double *A_OUT;
@@ -148,6 +149,8 @@ SEXP R_PDGESVD(SEXP M, SEXP N, SEXP ASIZE, SEXP A, SEXP DESCA, SEXP ALDIM,
   double temp_A = 0, temp_work = 0, *WORK;
 
   /* Protect R objects. */
+  PROTECT(A);
+  
   PROTECT(RET = allocVector(VECSXP, 4));
   PROTECT(RET_NAMES = allocVector(STRSXP, 4));
   
@@ -167,11 +170,7 @@ SEXP R_PDGESVD(SEXP M, SEXP N, SEXP ASIZE, SEXP A, SEXP DESCA, SEXP ALDIM,
   SET_STRING_ELT(RET_NAMES, 2, mkChar("u")); 
   SET_STRING_ELT(RET_NAMES, 3, mkChar("vt")); 
   setAttrib(RET, R_NamesSymbol, RET_NAMES);
-
-  /* Make copy of original data, since pdgesvd destroys it */
-  i = pt_ALDIM[0] * pt_ALDIM[1];
-  A_OUT = (double *) R_alloc(i, sizeof(double));
-  memcpy(A_OUT, REAL(A), i * sizeof(double));
+  
   
   /* Query size of workspace */
   INTEGER(INFO)[0] = 0;
@@ -189,20 +188,90 @@ SEXP R_PDGESVD(SEXP M, SEXP N, SEXP ASIZE, SEXP A, SEXP DESCA, SEXP ALDIM,
   WORK = (double *) R_alloc(temp_lwork, sizeof(double));
   
   INTEGER(INFO)[0] = 0;
-  F77_CALL(pdgesvd)(CHARPT(JOBU, 0), CHARPT(JOBVT, 0),
-    INTEGER(M), INTEGER(N),
-    A_OUT, &temp_IJ, &temp_IJ, INTEGER(DESCA),
-    REAL(D), REAL(U), &temp_IJ, &temp_IJ, INTEGER(DESCU),
-    REAL(VT), &temp_IJ, &temp_IJ, INTEGER(DESCVT),
-    WORK, &temp_lwork, INTEGER(INFO));
-
-  /* Return. */
-  UNPROTECT(6);
-
+  
+/*  if (CHARPT(INPLACE, 0)[0] == 'N'){*/
+    /* Make copy of original data, since pdgesvd destroys it */
+    i = pt_ALDIM[0] * pt_ALDIM[1];
+    A_OUT = (double *) R_alloc(i, sizeof(double));
+    memcpy(A_OUT, REAL(A), i * sizeof(double));
+    
+    pdgesvd_(CHARPT(JOBU, 0), CHARPT(JOBVT, 0),
+      INTEGER(M), INTEGER(N),
+      A_OUT, &temp_IJ, &temp_IJ, INTEGER(DESCA),
+      REAL(D), REAL(U), &temp_IJ, &temp_IJ, INTEGER(DESCU),
+      REAL(VT), &temp_IJ, &temp_IJ, INTEGER(DESCVT),
+      WORK, &temp_lwork, INTEGER(INFO));
+/*  }*/
+/*  else {*/
+/*    pdgesvd_(CHARPT(JOBU, 0), CHARPT(JOBVT, 0),*/
+/*      INTEGER(M), INTEGER(N),*/
+/*      REAL(A), &temp_IJ, &temp_IJ, INTEGER(DESCA),*/
+/*      REAL(D), REAL(U), &temp_IJ, &temp_IJ, INTEGER(DESCU),*/
+/*      REAL(VT), &temp_IJ, &temp_IJ, INTEGER(DESCVT),*/
+/*      WORK, &temp_lwork, INTEGER(INFO));*/
+/*  }*/
+  
+  UNPROTECT(7);
+  
   return(RET);
-} /* End of R_PDGESVD(). */
+} 
 
 
+/* Eigen */
+SEXP R_PDSYEV(SEXP JOBZ, SEXP UPLO, SEXP N, SEXP A, SEXP DESCA, SEXP ALDIM, SEXP ZLDIM, SEXP DESCZ)
+{
+  int i, *pt_ALDIM = INTEGER(ALDIM);
+  double *A_OUT;
+  SEXP RET, RET_NAMES, INFO, W, Z;
+  
+  /* Extra needed. */
+  int temp_IJ = 1, temp_lwork = -1;
+  double temp_A = 0, temp_work = 0, *WORK;
+  
+  /* Protect R objects. */
+  PROTECT(A);
+  
+  PROTECT(RET = allocVector(VECSXP, 3));
+  PROTECT(RET_NAMES = allocVector(STRSXP, 3));
+  
+  PROTECT(W = allocVector(REALSXP, INTEGER(N)[0]));
+  PROTECT(Z = allocMatrix(REALSXP, INTEGER(ZLDIM)[0], INTEGER(ZLDIM)[1]));
+  PROTECT(INFO = allocVector(INTSXP, 1));
+  
+  SET_VECTOR_ELT(RET, 0, W);
+  SET_VECTOR_ELT(RET, 1, Z);
+  SET_VECTOR_ELT(RET, 2, INFO);
+  
+  SET_STRING_ELT(RET_NAMES, 0, mkChar("values")); 
+  SET_STRING_ELT(RET_NAMES, 1, mkChar("vectors")); 
+  SET_STRING_ELT(RET_NAMES, 2, mkChar("info")); 
+  setAttrib(RET, R_NamesSymbol, RET_NAMES);
+  
+  /* Query size of workspace */
+  INTEGER(INFO)[0] = 0;
+  pdsyev_(CHARPT(JOBZ, 0), CHARPT(UPLO, 0), INTEGER(N),
+    &temp_A, &temp_IJ, &temp_IJ, INTEGER(DESCA),
+    &temp_A, &temp_A, &temp_IJ, &temp_IJ, INTEGER(DESCZ),
+    &temp_work, &temp_lwork, INTEGER(INFO));
+    
+  /* Allocate work vector and calculate svd */
+  temp_lwork = (int) temp_work;
+  temp_lwork = nonzero(temp_lwork);
+  
+  WORK = (double *) R_alloc(temp_lwork, sizeof(double));
+  
+  INTEGER(INFO)[0] = 0;
+  
+  pdsyev_(CHARPT(JOBZ, 0), CHARPT(UPLO, 0), INTEGER(N),
+    REAL(A), &temp_IJ, &temp_IJ, INTEGER(DESCA),
+    REAL(W), REAL(Z), &temp_IJ, &temp_IJ, INTEGER(DESCZ),
+    WORK, &temp_lwork, INTEGER(INFO));
+  
+  UNPROTECT(6);
+  
+  return(RET);
+} 
+
 
 /* LU factorization */
 SEXP R_PDGETRF(SEXP M, SEXP N, SEXP A, SEXP CLDIM, SEXP DESCA, SEXP LIPIV)
